==> crio-5.0.0.tgz: package/es/CrioObject.js <==
var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// external dependencies
import stringify from 'json-stringify-safe';
import { parse } from 'pathington';
import hashIt from 'hash-it';
import { get as _get, has as _has, merge as _merge, remove, set as _set } from 'unchanged';

// classes
import CrioArray from './CrioArray';

// constants
import { OBJECT_UNSCOPABLES } from './constants';

// is
import { isCrio, isEqual, isObject } from './is';

// utils
import { createIterator, getCrioedObject, getEntries, getValues, every as _every, find as _find, some as _some, thaw as _thaw } from './utils';

var hasAppliedPrototype = void 0;

var CrioObject = function () {
  function CrioObject(object) {
    _classCallCheck(this, CrioObject);

    var objectKeys = isObject(object) ? Object.keys(object) : [];

    if (!hasAppliedPrototype) {
      applyPrototype();

      hasAppliedPrototype = true;
    }

    if (isCrio(object)) {
      return object.toObject();
    }

    return objectKeys.reduce(function (crioObject, key) {
      crioObject[key] = getCrioedObject(object[key]);

      return crioObject;
    }, this);
  }

  /**
   * @function clear
   * @memberof CrioObject
   *
   * @description
   * get a new crio that is empty
   *
   * @returns {CrioObject} an empty object
   */
  CrioObject.prototype.clear = function clear() {
    return new CrioObject();
  };

  /**
   * @function compact
   * @memberof CrioObject
   *
   * @description
   * get a new object with values from the original array that are truthy
   *
   * @returns {CrioObject} the object with only truthy values
   */


  CrioObject.prototype.compact = function compact() {
    return this.filter(function (item) {
      return !!item;
    });
  };

  /**
   * @function delete
   * @memberof CrioObject
   *
   * @description
   * delete the value in the object at key, either shallow or deep
   *
   * @param {Array<number|string>|sring} key the key to delete
   * @returns {CrioObject} the array with the key deleted
   */


  CrioObject.prototype.delete = function _delete(key) {
    return remove(key, this);
  };

  /**
   * @function entries
   * @memberof CrioObject
   *
   * @description
   * get the pairs of [key, value] in the crio
   *
   * @returns {CrioArray} [key, value] pairs
   */


  CrioObject.prototype.entries = function entries() {
    return getEntries(this);
  };

  /**
   * @function equals
   * @memberof CrioObject
   *
   * @description
   * does the object passed equal the crio
   *
   * @param {*} object object to compare against the instance
   * @returns {boolean} is the object equivalent in value
   */


  CrioObject.prototype.equals = function equals(object) {
    return isEqual(this, object);
  };

  /**
   * @function every
   * @memberof CrioObject
   *
   * @description
   * does every instance in the object match
   *
   * @param {function} fn the function to test for matching
   * @returns {boolean} does every instance match
   */


  CrioObject.prototype.every = function every(fn) {
    return _every(this, fn);
  };

  /**
   * @function filter
   * @memberof CrioObject
   *
   * @description
   * filter the object based on the fn passed
   *
   * @param {function} fn function to test for if it should be included in the result set
   * @returns {CrioObject} new crio instance
   */


  CrioObject.prototype.filter = function filter(fn) {
    var _this = this;

    return new CrioObject(Object.keys(this).reduce(function (object, key) {
      if (fn(_this[key], key, object)) {
        object[key] = _this[key];
      }

      return object;
    }, {}));
  };

  /**
   * @function find
   * @memberof CrioObject
   *
   * @description
   * find an item in the crio if it exists
   *
   * @param {function} fn function to test for finding the item
   * @returns {*} found item or undefined
   */


  CrioObject.prototype.find = function find(fn) {
    return _find(this, fn);
  };

  /**
   * @function findKey
   * @memberof CrioObject
   *
   * @description
   * find the key of an item in the crio if it exists
   *
   * @param {function} fn function to test for finding the item
   * @returns {number} index of match, or -1
   */


  CrioObject.prototype.findKey = function findKey(fn) {
    return _find(this, fn, true);
  };

  /**
   * @function findLast
   * @memberof CrioObject
   *
   * @description
   * find an item in the crio if it exists, starting from the end and iteratng to the start
   *
   * @param {function} fn function to test for finding the item
   * @returns {*} found item or undefined
   */


  CrioObject.prototype.findLast = function findLast(fn) {
    return _find(this, fn, false, true);
  };

  /**
   * @function findLastKey
   * @memberof CrioObject
   *
   * @description
   * find the matching index based on truthy return from fn starting from end
   *
   * @param {function} fn function to use for test in iteration
   * @returns {number} index of match, or -1
   */


  CrioObject.prototype.findLastKey = function findLastKey(fn) {
    return _find(this, fn, true, true);
  };

  /**
   * @function forEach
   * @memberof CrioObject
   *
   * @description
   * iterate over the object calling fn
   *
   * @param {function} fn function to call in iteration
   * @returns {CrioObject} the original object
   */


  CrioObject.prototype.forEach = function forEach(fn) {
    var _this2 = this;

    Object.keys(this).forEach(function (key) {
      fn(_this2[key], key, _this2);
    });

    return this;
  };

  /**
   * @function get
   * @memberof CrioObject
   *
   * @description
   * get the item at key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|string} key key to retrieve
   * @returns {*} item found at key
   */


  CrioObject.prototype.get = function get(key) {
    return _get(key, this);
  };

  /**
   * @function has
   * @memberof CrioObject
   *
   * @description
   * does the crio have the key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|string} key key to test
   * @returns {boolean} does the crio have the key
   */


  CrioObject.prototype.has = function has(key) {
    return _has(key, this);
  };

  /**
   * @function includes
   * @memberof CrioObject
   *
   * @description
   * does the object have the item passed
   *
   * @param {*} item item to test for existence
   * @returns {boolean} does the item exist in the crio
   */


  CrioObject.prototype.includes = function includes(item) {
    return this.some(function (value) {
      return value === item;
    });
  };

  /**
   * @function isArray
   * @memberof CrioObject
   *
   * @description
   * is the crio an array
   *
   * @returns {boolean} is the crio an array
   */


  CrioObject.prototype.isArray = function isArray() {
    return false;
  };

  /**
   * @function isObject
   * @memberof CrioObject
   *
   * @description
   * is the crio an object
   *
   * @returns {boolean} is the crio an object
   */


  CrioObject.prototype.isObject = function isObject() {
    return true;
  };

  /**
   * @function keyOf
   * @memberof CrioObject
   *
   * @description
   * get the key for the item passed
   *
   * @param {*} item the item to search for
   * @returns {string} the key of match, or undefined
   */


  CrioObject.prototype.keyOf = function keyOf(item) {
    return this.findKey(function (value) {
      return value === item;
    });
  };

  /**
   * @function keys
   * @memberof CrioObject
   *
   * @description
   * get the keys of the crio
   *
   * @returns {CrioArray<string>} keys in the crio
   */


  CrioObject.prototype.keys = function keys() {
    return new CrioArray(Object.keys(this));
  };

  /**
   * @function lastKeyOf
   * @memberof CrioObject
   *
   * @description
   * get the key for the item passed, starting from the end of the array and iterating towards the start
   *
   * @param {*} item the item to search for
   * @returns {string} the key of match, or undefined
   */


  CrioObject.prototype.lastKeyOf = function lastKeyOf(item) {
    return this.findLastKey(function (value) {
      return value === item;
    });
  };

  /**
   * @function map
   * @memberof CrioObject
   *
   * @description
   * iterate over the object mapping the result of fn to the key
   *
   * @param {function} fn function to call on iteration
   * @returns {Crio} the mapped object
   */


  CrioObject.prototype.map = function map(fn) {
    var _this3 = this;

    return Object.keys(this).reduce(function (object, key) {
      object[key] = getCrioedObject(fn(_this3[key], key, _this3));

      return object;
    }, new CrioObject({}));
  };

  /**
   * @function merge
   * @memberof CrioObject
   *
   * @description
   * merge objects with the original object
   *
   * @param {Array<number|string>|number|null} key the key to merge into
   * @param {...Array<CrioObject>} objects objects to merge with the crio
   * @returns {CrioObject} new crio instance
   */


  CrioObject.prototype.merge = function merge(key) {
    for (var _len = arguments.length, objects = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      objects[_key - 1] = arguments[_key];
    }

    return objects.reduce(function (mergedObject, object) {
      return _merge(key, getCrioedObject(object), mergedObject);
    }, this);
  };

  /**
   * @function mutate
   * @memberof CrioObject
   *
   * @description
   * work with the object in a mutated way and return the crioed result of that call
   *
   * @param {function} fn function to apply to crio
   * @returns {*} crioed value resulting from the call
   */


  CrioObject.prototype.mutate = function mutate(fn) {
    return getCrioedObject(fn(this.thaw(), this));
  };

  /**
   * @function pluck
   * @memberof CrioObject
   *
   * @description
   * get the values in each object in the collection at key, either shallow or deeply nested
   *
   * @param {string} key key to find value of in collection object
   * @returns {CrioArray} array of plucked values
   */


  CrioObject.prototype.pluck = function pluck(key) {
    var parsedKey = parse(key);

    var objectToPluck = _get(parsedKey.slice(0, parsedKey.length - 1), this);
    var finalKey = parsedKey.slice(-1);

    return objectToPluck.map(function (item) {
      return _get(finalKey, item);
    }).values();
  };

  /**
   * @function reduce
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioObject.prototype.reduce = function reduce(fn, initialValue) {
    var _this4 = this;

    return getCrioedObject(Object.keys(this).reduce(function (value, key) {
      return fn(value, _this4[key], key, _this4);
    }, initialValue));
  };

  /**
   * @function reduceRight
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value, starting from the end of the array
   * and iterating to the start
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioObject.prototype.reduceRight = function reduceRight(fn, initialValue) {
    var _this5 = this;

    return getCrioedObject(Object.keys(this).reverse().reduce(function (value, key) {
      return fn(value, _this5[key], key, _this5);
    }, initialValue));
  };

  /**
   * @function set
   * @memberof CrioObject
   *
   * @description
   * set the value at the key passed
   *
   * @param {Array<number|string>|string} key key to assign value to
   * @param {*} value value to assign
   * @returns {CrioObject} object with value set at key
   */


  CrioObject.prototype.set = function set(key, value) {
    return _set(key, getCrioedObject(value), this);
  };

  /**
   * @function some
   * @memberof CrioObject
   *
   * @description
   * does any item in the object match the result from fn
   *
   * @param {function} fn the function to test for matching
   * @returns {boolean} does any item match
   */


  CrioObject.prototype.some = function some(fn) {
    return _some(this, fn);
  };

  /**
   * @function sort
   * @memberof CrioObject
   *
   * @description
   * sort the collection by the fn passed
   *
   * @param {function} fn the function to sort based on
   * @returns {CrioObject} object with the items sorted
   */


  CrioObject.prototype.sort = function sort(fn) {
    var _this6 = this;

    return new CrioObject(Object.keys(this).sort(fn).reduce(function (object, key) {
      object[key] = _this6[key];

      return object;
    }, {}));
  };

  /**
   * @function thaw
   * @memberof CrioObject
   *
   * @description
   * create a plain JS version of the object
   *
   * @returns {Object} plain JS version of the object
   */


  CrioObject.prototype.thaw = function thaw() {
    return _thaw(this);
  };

  /**
   * @function toArray
   * @memberof CrioObject
   *
   * @description
   * convert the object to an array
   *
   * @returns {CrioArray} the object converted to an array of its values
   */


  CrioObject.prototype.toArray = function toArray() {
    return this.values();
  };

  /**
   * @function toLocaleString
   * @memberof CrioObject
   *
   * @description
   * convert the object to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified object
   */


  CrioObject.prototype.toLocaleString = function toLocaleString(serializer, indent) {
    return this.toString(serializer, indent);
  };

  /**
   * @function toObject
   * @memberof CrioObject
   *
   * @description
   * convert the object to an objectobject
   *
   * @returns {CrioObject} the object
   */


  CrioObject.prototype.toObject = function toObject() {
    return this;
  };

  /**
   * @function toString
   * @memberof CrioObject
   *
   * @description
   * convert the object to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified object
   */


  CrioObject.prototype.toString = function toString(serializer, indent) {
    return stringify(this, serializer, indent);
  };

  /**
   * @function valueOf
   * @memberof CrioObject
   *
   * @description
   * get the object value
   *
   * @returns {CrioObject} the object
   */


  CrioObject.prototype.valueOf = function valueOf() {
    return this;
  };

  /**
   * @function values
   * @memberof CrioObject
   *
   * @description
   * get the values of the object as an array
   *
   * @returns {CrioObject} values in the object
   */


  CrioObject.prototype.values = function values() {
    return getValues(this);
  };

  _createClass(CrioObject, [{
    key: 'hashCode',
    get: function get() {
      return hashIt(this, true);
    }
  }, {
    key: 'size',
    get: function get() {
      return Object.keys(this).length;
    }
  }]);

  return CrioObject;
}();

export function applyPrototype() {
  if (typeof Symbol === 'function') {
    if (Symbol.species) {
      Object.defineProperty(CrioObject, Symbol.species, {
        configurable: false,
        enumerable: false,
        get: function get() {
          return CrioObject;
        }
      });
    }

    if (Symbol.iterator) {
      Object.defineProperty(CrioObject.prototype, Symbol.iterator, {
        configurable: false,
        enumerable: false,
        value: createIterator(),
        writable: false
      });
    }

    if (Symbol.unscopables) {
      Object.defineProperty(CrioObject.prototype, Symbol.unscopables, {
        configurable: false,
        enumerable: false,
        value: OBJECT_UNSCOPABLES,
        writable: false
      });
    }
  }
}

export default CrioObject;

==> crio-5.0.2.tgz: package/es/CrioObject.js <==
var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// external dependencies
import stringify from 'json-stringify-safe';
import { parse } from 'pathington';
import hashIt from 'hash-it';
import { get as _get, has as _has, merge as _merge, remove, set as _set } from 'unchanged';

// classes
import CrioArray from './CrioArray';

// constants
import { OBJECT_UNSCOPABLES } from './constants';

// is
import { isCrio, isEqual, isObject } from './is';

// utils
import { createIterator, getCrioedObject, getEntries, getValues, every as _every, find as _find, some as _some, thaw as _thaw } from './utils';

var hasAppliedPrototype = void 0;

var CrioObject = function () {
  function CrioObject(object) {
    _classCallCheck(this, CrioObject);

    if (!hasAppliedPrototype) {
      applyPrototype(); // eslint-disable-line no-use-before-define

      hasAppliedPrototype = true;
    }

    if (isCrio(object)) {
      return object.toObject();
    }

    var objectKeys = isObject(object) ? Object.keys(object) : [];

    return objectKeys.reduce(function (crioObject, key) {
      crioObject[key] = getCrioedObject(object[key]);

      return crioObject;
    }, this);
  }

  /**
   * @function clear
   * @memberof CrioObject
   *
   * @description
   * get a new crio that is empty
   *
   * @returns {CrioObject} an empty object
   */
  CrioObject.prototype.clear = function clear() {
    return new CrioObject();
  };

  /**
   * @function compact
   * @memberof CrioObject
   *
   * @description
   * get a new object with values from the original array that are truthy
   *
   * @returns {CrioObject} the object with only truthy values
   */


  CrioObject.prototype.compact = function compact() {
    return this.filter(function (item) {
      return !!item;
    });
  };

  /**
   * @function delete
   * @memberof CrioObject
   *
   * @description
   * delete the value in the object at key, either shallow or deep
   *
   * @param {Array<number|string>|sring} key the key to delete
   * @returns {CrioObject} the array with the key deleted
   */


  CrioObject.prototype.delete = function _delete(key) {
    return remove(key, this);
  };

  /**
   * @function entries
   * @memberof CrioObject
   *
   * @description
   * get the pairs of [key, value] in the crio
   *
   * @returns {CrioArray} [key, value] pairs
   */


  CrioObject.prototype.entries = function entries() {
    return getEntries(this);
  };

  /**
   * @function equals
   * @memberof CrioObject
   *
   * @description
   * does the object passed equal the crio
   *
   * @param {*} object object to compare against the instance
   * @returns {boolean} is the object equivalent in value
   */


  CrioObject.prototype.equals = function equals(object) {
    return isEqual(this, object);
  };

  /**
   * @function every
   * @memberof CrioObject
   *
   * @description
   * does every instance in the object match
   *
   * @param {function} fn the function to test for matching
   * @returns {boolean} does every instance match
   */


  CrioObject.prototype.every = function every(fn) {
    return _every(this, fn);
  };

  /**
   * @function filter
   * @memberof CrioObject
   *
   * @description
   * filter the object based on the fn passed
   *
   * @param {function} fn function to test for if it should be included in the result set
   * @returns {CrioObject} new crio instance
   */


  CrioObject.prototype.filter = function filter(fn) {
    var _this = this;

    return new CrioObject(Object.keys(this).reduce(function (object, key) {
      if (fn(_this[key], key, object)) {
        object[key] = _this[key];
      }

      return object;
    }, {}));
  };

  /**
   * @function find
   * @memberof CrioObject
   *
   * @description
   * find an item in the crio if it exists
   *
   * @param {function} fn function to test for finding the item
   * @returns {*} found item or undefined
   */


  CrioObject.prototype.find = function find(fn) {
    return _find(this, fn);
  };

  /**
   * @function findKey
   * @memberof CrioObject
   *
   * @description
   * find the key of an item in the crio if it exists
   *
   * @param {function} fn function to test for finding the item
   * @returns {number} index of match, or -1
   */


  CrioObject.prototype.findKey = function findKey(fn) {
    return _find(this, fn, true);
  };

  /**
   * @function findLast
   * @memberof CrioObject
   *
   * @description
   * find an item in the crio if it exists, starting from the end and iteratng to the start
   *
   * @param {function} fn function to test for finding the item
   * @returns {*} found item or undefined
   */


  CrioObject.prototype.findLast = function findLast(fn) {
    return _find(this, fn, false, true);
  };

  /**
   * @function findLastKey
   * @memberof CrioObject
   *
   * @description
   * find the matching index based on truthy return from fn starting from end
   *
   * @param {function} fn function to use for test in iteration
   * @returns {number} index of match, or -1
   */


  CrioObject.prototype.findLastKey = function findLastKey(fn) {
    return _find(this, fn, true, true);
  };

  /**
   * @function forEach
   * @memberof CrioObject
   *
   * @description
   * iterate over the object calling fn
   *
   * @param {function} fn function to call in iteration
   * @returns {CrioObject} the original object
   */


  CrioObject.prototype.forEach = function forEach(fn) {
    var _this2 = this;

    Object.keys(this).forEach(function (key) {
      return fn(_this2[key], key, _this2);
    });

    return this;
  };

  /**
   * @function get
   * @memberof CrioObject
   *
   * @description
   * get the item at key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|string} key key to retrieve
   * @returns {*} item found at key
   */


  CrioObject.prototype.get = function get(key) {
    return _get(key, this);
  };

  /**
   * @function has
   * @memberof CrioObject
   *
   * @description
   * does the crio have the key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|string} key key to test
   * @returns {boolean} does the crio have the key
   */


  CrioObject.prototype.has = function has(key) {
    return _has(key, this);
  };

  /**
   * @function includes
   * @memberof CrioObject
   *
   * @description
   * does the object have the item passed
   *
   * @param {*} item item to test for existence
   * @returns {boolean} does the item exist in the crio
   */


  CrioObject.prototype.includes = function includes(item) {
    return this.some(function (value) {
      return value === item;
    });
  };

  /**
   * @function isArray
   * @memberof CrioObject
   *
   * @description
   * is the crio an array
   *
   * @returns {boolean} is the crio an array
   */


  CrioObject.prototype.isArray = function isArray() {
    return false;
  };

  /**
   * @function isObject
   * @memberof CrioObject
   *
   * @description
   * is the crio an object
   *
   * @returns {boolean} is the crio an object
   */


  CrioObject.prototype.isObject = function isObject() {
    return true;
  };

  /**
   * @function keyOf
   * @memberof CrioObject
   *
   * @description
   * get the key for the item passed
   *
   * @param {*} item the item to search for
   * @returns {string} the key of match, or undefined
   */


  CrioObject.prototype.keyOf = function keyOf(item) {
    return this.findKey(function (value) {
      return value === item;
    });
  };

  /**
   * @function keys
   * @memberof CrioObject
   *
   * @description
   * get the keys of the crio
   *
   * @returns {CrioArray<string>} keys in the crio
   */


  CrioObject.prototype.keys = function keys() {
    return new CrioArray(Object.keys(this));
  };

  /**
   * @function lastKeyOf
   * @memberof CrioObject
   *
   * @description
   * get the key for the item passed, starting from the end of the array and iterating towards the start
   *
   * @param {*} item the item to search for
   * @returns {string} the key of match, or undefined
   */


  CrioObject.prototype.lastKeyOf = function lastKeyOf(item) {
    return this.findLastKey(function (value) {
      return value === item;
    });
  };

  /**
   * @function map
   * @memberof CrioObject
   *
   * @description
   * iterate over the object mapping the result of fn to the key
   *
   * @param {function} fn function to call on iteration
   * @returns {Crio} the mapped object
   */


  CrioObject.prototype.map = function map(fn) {
    var _this3 = this;

    return Object.keys(this).reduce(function (object, key) {
      object[key] = getCrioedObject(fn(_this3[key], key, _this3));

      return object;
    }, new CrioObject({}));
  };

  /**
   * @function merge
   * @memberof CrioObject
   *
   * @description
   * merge objects with the original object
   *
   * @param {Array<number|string>|number|null} key the key to merge into
   * @param {...Array<CrioObject>} objects objects to merge with the crio
   * @returns {CrioObject} new crio instance
   */


  CrioObject.prototype.merge = function merge(key) {
    for (var _len = arguments.length, objects = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      objects[_key - 1] = arguments[_key];
    }

    return objects.reduce(function (mergedObject, object) {
      return _merge(key, getCrioedObject(object), mergedObject);
    }, this);
  };

  /**
   * @function mutate
   * @memberof CrioObject
   *
   * @description
   * work with the object in a mutated way and return the crioed result of that call
   *
   * @param {function} fn function to apply to crio
   * @returns {*} crioed value resulting from the call
   */


  CrioObject.prototype.mutate = function mutate(fn) {
    return getCrioedObject(fn(this.thaw(), this));
  };

  /**
   * @function pluck
   * @memberof CrioObject
   *
   * @description
   * get the values in each object in the collection at key, either shallow or deeply nested
   *
   * @param {string} key key to find value of in collection object
   * @returns {CrioArray} array of plucked values
   */


  CrioObject.prototype.pluck = function pluck(key) {
    var parsedKey = parse(key);
    var objectToPluck = _get(parsedKey.slice(0, parsedKey.length - 1), this);
    var finalKey = parsedKey.slice(-1);

    return objectToPluck.map(function (item) {
      return _get(finalKey, item);
    }).values();
  };

  /**
   * @function reduce
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioObject.prototype.reduce = function reduce(fn, initialValue) {
    var _this4 = this;

    return getCrioedObject(Object.keys(this).reduce(function (value, key) {
      return fn(value, _this4[key], key, _this4);
    }, initialValue));
  };

  /**
   * @function reduceRight
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value, starting from the end of the array
   * and iterating to the start
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioObject.prototype.reduceRight = function reduceRight(fn, initialValue) {
    var _this5 = this;

    return getCrioedObject(Object.keys(this).reduceRight(function (value, key) {
      return fn(value, _this5[key], key, _this5);
    }, initialValue));
  };

  /**
   * @function set
   * @memberof CrioObject
   *
   * @description
   * set the value at the key passed
   *
   * @param {Array<number|string>|string} key key to assign value to
   * @param {*} value value to assign
   * @returns {CrioObject} object with value set at key
   */


  CrioObject.prototype.set = function set(key, value) {
    return _set(key, getCrioedObject(value), this);
  };

  /**
   * @function some
   * @memberof CrioObject
   *
   * @description
   * does any item in the object match the result from fn
   *
   * @param {function} fn the function to test for matching
   * @returns {boolean} does any item match
   */


  CrioObject.prototype.some = function some(fn) {
    return _some(this, fn);
  };

  /**
   * @function sort
   * @memberof CrioObject
   *
   * @description
   * sort the collection by the fn passed
   *
   * @param {function} fn the function to sort based on
   * @returns {CrioObject} object with the items sorted
   */


  CrioObject.prototype.sort = function sort(fn) {
    var _this6 = this;

    return new CrioObject(Object.keys(this).sort(fn).reduce(function (object, key) {
      object[key] = _this6[key];

      return object;
    }, {}));
  };

  /**
   * @function thaw
   * @memberof CrioObject
   *
   * @description
   * create a plain JS version of the object
   *
   * @returns {Object} plain JS version of the object
   */


  CrioObject.prototype.thaw = function thaw() {
    return _thaw(this);
  };

  /**
   * @function toArray
   * @memberof CrioObject
   *
   * @description
   * convert the object to an array
   *
   * @returns {CrioArray} the object converted to an array of its values
   */


  CrioObject.prototype.toArray = function toArray() {
    return this.values();
  };

  /**
   * @function toLocaleString
   * @memberof CrioObject
   *
   * @description
   * convert the object to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified object
   */


  CrioObject.prototype.toLocaleString = function toLocaleString(serializer, indent) {
    return this.toString(serializer, indent);
  };

  /**
   * @function toObject
   * @memberof CrioObject
   *
   * @description
   * convert the object to an objectobject
   *
   * @returns {CrioObject} the object
   */


  CrioObject.prototype.toObject = function toObject() {
    return this;
  };

  /**
   * @function toString
   * @memberof CrioObject
   *
   * @description
   * convert the object to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified object
   */


  CrioObject.prototype.toString = function toString(serializer, indent) {
    return stringify(this, serializer, indent);
  };

  /**
   * @function valueOf
   * @memberof CrioObject
   *
   * @description
   * get the object value
   *
   * @returns {CrioObject} the object
   */


  CrioObject.prototype.valueOf = function valueOf() {
    return this;
  };

  /**
   * @function values
   * @memberof CrioObject
   *
   * @description
   * get the values of the object as an array
   *
   * @returns {CrioObject} values in the object
   */


  CrioObject.prototype.values = function values() {
    return getValues(this);
  };

  _createClass(CrioObject, [{
    key: 'hashCode',
    get: function get() {
      return hashIt(this, true);
    }
  }, {
    key: 'size',
    get: function get() {
      return Object.keys(this).length;
    }
  }]);

  return CrioObject;
}();

export function applyPrototype() {
  if (typeof Symbol === 'function') {
    if (Symbol.species) {
      Object.defineProperty(CrioObject, Symbol.species, {
        configurable: false,
        enumerable: false,
        get: function get() {
          return CrioObject;
        }
      });
    }

    if (Symbol.iterator) {
      Object.defineProperty(CrioObject.prototype, Symbol.iterator, {
        configurable: false,
        enumerable: false,
        value: createIterator(),
        writable: false
      });
    }

    if (Symbol.unscopables) {
      Object.defineProperty(CrioObject.prototype, Symbol.unscopables, {
        configurable: false,
        enumerable: false,
        value: OBJECT_UNSCOPABLES,
        writable: false
      });
    }
  }
}

export default CrioObject;
